/*
Pluck: an open source trivia game for Android

Copyright (C) 2021 Joguitos do luca0N!

This program is free software: you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free Software
Foundation, either version 3 of the License, or (at your option) any later
version.

This program is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with
this program. If not, see <https://www.gnu.org/licenses/>.

This game is a fork of LibreTrivia, and its source code is available at
<https://github.com/tryton-vanmeer/LibreTrivia>.

Contact us at <joguitos+pluck@luca0n.com>.
*/

package com.luca0n.joguitos.pluck.trivia;

import com.luca0n.joguitos.pluck.PluckApplication;
import com.luca0n.joguitos.pluck.R;

import java.io.Serializable;

import android.content.Context;
import android.content.SharedPreferences;
import android.preference.PreferenceManager;

public class TriviaQuery implements Serializable {
	private static final int DEFAULT_AMOUNT = 10;

	private final int amount;
	private final TriviaCategory category;
	private final TriviaDifficulty difficulty;
	private final TriviaType type;

	private TriviaQuery(Builder builder) {
		this.amount = builder.amount;
		this.category = builder.category;
		this.difficulty = builder.difficulty;
		this.type = builder.type;
	}

	public static class Builder {
		private final int amount;
		private TriviaCategory category;
		private TriviaDifficulty difficulty;
		private TriviaType type;

		public Builder() {
			this.amount = DEFAULT_AMOUNT;
		}

		public Builder(int amount) {
			if (amount > 50) {
				this.amount = 50;
			} else {
				this.amount = amount;
			}
		}

		public Builder category(TriviaCategory category) {
			this.category = category;
			return this;
		}

		public Builder difficulty(TriviaDifficulty difficulty) {
			this.difficulty = difficulty;
			return this;
		}

		public Builder type(TriviaType type) {
			this.type = type;
			return this;
		}

		public TriviaQuery build() {
			return new TriviaQuery(this);
		}
	}

	@Override
	public String toString() {
		StringBuilder url = new StringBuilder();

		// Load custom server URL.
		Context c = PluckApplication.getAppContext();
		SharedPreferences sp = PreferenceManager.getDefaultSharedPreferences(c);
		String serverKey = c.getResources().getString(R.string.pref_network_server);
		// Load default server URL in case the player did not specify a custom server URL.
		String defaultServer = c.getResources().getString(R.string.pref_network_server_default);
		String base = sp.getString(serverKey, defaultServer) + "?"; // append "?" to the end of the URL so we can specify GET parameters.

		// Check if the custom server URL preference was empty. If it was, change it to the default URL.
		if (base.equals("?")){
			base = defaultServer + "?";
			sp.edit().putString(serverKey, defaultServer).commit();
		}

		url.append(base);
		url.append("amount=").append(this.amount);

		if (this.category != null & this.category != TriviaCategory.ANY) {
			url.append("&category=").append(this.category.getID());
		}
		if (this.difficulty != null & this.difficulty != TriviaDifficulty.ANY) {
			url.append("&difficulty=").append(this.difficulty.getName());
		}
		if (this.type != null & this.type != TriviaType.ANY) {
			url.append("&type=").append(this.type.getName());
		}

		return url.toString();
	}
}
